CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  eeprom_pin = 28

  file_info_serial = 1
  file_info_fileio = 2
  
  file_info_type   = 0
  file_info_data   = 1
  file_info_size   = 8

OBJ
  rendez : "rendezvous"
  fileio : "cfileio_os"
  mem : "cmalloc"
  ser : "cserial"
  serp : "cserialp"
  spi_p : "safe_spi_p"

DAT
  sdpin byte 0[4]
  pins byte "DO ", 0, "CLK", 0, "DI ", 0, "CS ", 0

PUB main | i, index, num, argv[2]
  longfill(rendez#start, 0, ($8000-rendez#start) >> 2)
  start1(31, 30, %10000, 57600, 200)
  AddToCogTable(0, 1, string("bootsd.binary"), 0, long[rendez#memfreelist])
  AddToCogTable(1, 2, string("serial I/O"), 0, 0)

  repeat
    ser.dbprintf0(string("Press ESC for setup or any other key to start\n"))
    if ((i := ser.rxtime(1000)) <> -1)
      quit

  if long[@sdpin] == 0 OR i == 27
    repeat index from 0 to 3
      ser.dbprintf1(string("Enter %s pin number: "), @pins + (index << 2))
      num := 0
      repeat
        i := ser.rx
        ser.tx(i)
        if i < "0" OR i > "9"
          quit
        num := (num * 10) + i - "0"
      sdpin[index] := num
    WriteEEPromLong(eeprom_pin, @sdpin)

  ' Start the SPI Driver
  num := spi_p.start_explicit(sdpin[0], sdpin[1], sdpin[2], sdpin[3])
  AddToCogTable(2, 2, string("SPI Driver"), 0, 0)

  StartKernel

  if fileio.mount_explicit(sdpin[0], sdpin[1], sdpin[2], sdpin[3]) =< 0
    ser.dbprintf0(STRING("\nmount failed.\n"))
    repeat
  fileio.chdir(string("/"))

  argv[0] := string("shell")
  argv[1] := string("boot")
  fileio.run(2, @argv, 1)

  repeat

PUB AddToCogTable(cognum, type, name, startaddr, size) | entryptr
  entryptr := rendez#cogtable + (cognum * 24)
  byte[entryptr] := type
  bytemove(entryptr+1, name, 19)
  byte[entryptr+19] := 0
  word[entryptr+20] := startaddr
  word[entryptr+22] := size

DAT
  fileio1 FILE "kernel.binary"

PUB StartKernel | AppAddr, size, i, cognum
  size := word[@fileio1 + $a] ' Start of stack
  ' Allocate the memory plus some for the stack
  AppAddr := mem.calloc(size + 800)
  ifnot AppAddr
    ser.dbprintf0(string("Not enough memory to run file I/O\n"))
    return
  bytemove(AppAddr, @fileio1, size)
  ' Fixup
  repeat i from 0 to 4
    word[AppAddr+6+i<<1] += AppAddr
  ' Fill the stack with the check word
  longfill(AppAddr + size, rendez#checkword, 200)
  long[AppAddr + size]~
  cognum := cognew($f004, AppAddr + 4)
  AddToCogTable(cognum, 1, string("kernel.binary"), AppAddr, size + 800)

PUB start1(rxpin, txpin, mode, baudrate, stacksize) | stdio, stdin, stdout
{{
  This routine calls the malloc initialization routine.  The number of longs in
  the top object stack is set by the value of "stacksize".  It also calls the
  openserial function to initialize the standard I/O serial port.  It returns the
  stdio pointer if successful, or NULL if not.
}}
  mem.mallocinit1(stacksize, rendez#start)
  stdio := openserial(rxpin, txpin, mode, baudrate, 512, 64)
  stdin := mem.malloc(file_info_size)
  stdout := mem.malloc(file_info_size)
  long[rendez#stdio] := stdio
  long[rendez#stdin] := stdin
  long[rendez#stdout] := stdout
  bytemove(stdin, stdio, file_info_size)
  bytemove(stdout, stdio, file_info_size)
  return stdio

PUB openserial(rxpin, txpin, mode, baudrate, rxsize, txsize) | pfileinfo, file_data
{{
  This routine allocates memory for a file info and serial port data structure.
  It calls the start1 function in the cserial object to create a serial port in
  the next available cog.  It returns a pointer to the file info structure if
  successful, or a NULL value if not successful.
}}
  pfileinfo := mem.malloc(file_info_size + ser#header_size + rxsize + txsize)
  ifnot pfileinfo
    return 0
  file_data := pfileinfo + file_info_size
  long[pfileinfo][file_info_type] := file_info_serial
  long[pfileinfo][file_info_data] := file_data
  ifnot serp.start1(file_data, rxpin, txpin, mode, baudrate, rxsize, txsize)
    mem.free(pfileinfo)
    return 0
  return pfileinfo

' I2C EEProm methods follow below
PUB WriteEEPromLong(SCL, addr) | devSel, ackbit
  ' Make sure EEProm is in a known state
  Initialize(SCL)
  ' Write three address bytes
  devSel := $A0 | ((addr >> 15) & %1110)
  Start(SCL)
  Write(SCL, devSel)
  Write(SCL, addr >> 8 & $FF)
  Write(SCL, addr & $FF)
  ' Write four data bytes          
  repeat 4
    Write(SCL, byte[addr++])
  Stop(SCL)
  ' Wait till the write is done
  repeat
    Start(SCL)
    ackbit := Write(SCL, devSel)
    Stop(SCL)
  while ackbit

PUB Initialize(SCL) | SDA           ' An I2C device may be left in an
  SDA := SCL + 1                    '  invalid state and may need to be
  outa[SCL] := 1                    '   reinitialized.  Drive SCL high.
  dira[SCL] := 1
  dira[SDA] := 0                    ' Set SDA as input
  repeat 9
    outa[SCL] := 0                  ' Put out up to 9 clock pulses
    outa[SCL] := 1
    if ina[SDA]                     ' Repeat if SDA not driven high
      quit                          '  by the EEPROM

PUB Start(SCL) | SDA                ' SDA goes HIGH to LOW with SCL HIGH
  SDA := SCL + 1
  outa[SCL]~~                       ' Initially drive SCL HIGH
  dira[SCL]~~
  outa[SDA]~~                       ' Initially drive SDA HIGH
  dira[SDA]~~
  outa[SDA]~                        ' Now drive SDA LOW
  outa[SCL]~                        ' Leave SCL LOW
  
PUB Stop(SCL) | SDA                 ' SDA goes LOW to HIGH with SCL High
  SDA := SCL + 1
  outa[SCL]~~                       ' Drive SCL HIGH
  outa[SDA]~~                       '  then SDA HIGH
  dira[SCL]~                        ' Now let them float
  dira[SDA]~                        ' If pullups present, they'll stay HIGH

PUB Write(SCL, data) : ackbit | SDA
  SDA := SCL + 1
  ackbit := 0 
  data <<= 24
  repeat 8                          ' Output data to SDA
    outa[SDA] := (data <-= 1) & 1   
    outa[SCL]~~                     ' Toggle SCL from LOW to HIGH to LOW
    outa[SCL]~
  dira[SDA]~                        ' Set SDA to input for ACK/NAK
  outa[SCL]~~
  ackbit := ina[SDA]                ' Sample SDA when SCL is HIGH
  outa[SCL]~
  outa[SDA]~                        ' Leave SDA driven LOW
  dira[SDA]~~